@@ -7,15 +7,12 @@ from version import email as __email__
 from version import license_ as __license__
 from version import doc as __doc__
 
-from recipes import Recipe
-from parameters import Parameter
+from recipe import Recipe
+from param import Parameter
 from frames import FrameConfig
 from result import Result, CplError, RecipeCrash
-from log import msg, lib_version, lib_description
 import dfs
 import esorex
 
-msg.domain = os.path.basename(sys.argv[0])
-msg.level = msg.OFF
-
 Recipe.dir = '.'
+cpl_versions = [ '%i.%i.%i' % ver for ver in CPL_recipe.cpl_versions() ]
