@@ -104,7 +104,7 @@ class CplError(StandardError):
 
     The exception is raised on recipe invocation, or when accessing the result
     frames if the recipe was started in background
-    (:attr:`cpl.Recipe.threaded` set to :attr:`True`).
+    (:attr:`cpl.Recipe.threaded` set to :obj:`True`).
 
     Attributes:
 
@@ -128,11 +128,11 @@ class CplError(StandardError):
 
        Log lines of the recipe that lead to this exception.
 
-       .. seealso:: :class:`cpl.log.LogList`
+       .. seealso:: :class:`cpl.logger.LogList`
 
     .. attribute:: next
      
-       Next error, or :attr:`None`.
+       Next error, or :obj:`None`.
 
     '''
     def __init__(self, retval, res, logger = None):
@@ -164,12 +164,12 @@ class CplError(StandardError):
         if self.code is None:
             s = 'Unspecified'
         else:
-            s = "%s (%i) in %s() (%s:%s)" % (self.msg, self.code, 
+            s = "%s (%i) in %s() (%s:%i)" % (self.msg, self.code, 
                                              self.function, self.file, 
                                              self.line) 
         if self.next:
             for e in self.next:
-                s += "\n    %s (%i) in %s() (%s:%s)" % (e.msg, e.code, 
+                s += "\n    %s (%i) in %s() (%s:%i)" % (e.msg, e.code, 
                                                         e.function, e.file, 
                                                         e.line) 
         return s
@@ -187,7 +187,7 @@ class RecipeCrash(StandardError):
 
     The exception is raised on recipe invocation, or when accessing the result
     frames if the recipe was started in background
-    (:attr:`cpl.Recipe.threaded` set to :attr:`True`).
+    (:attr:`cpl.Recipe.threaded` set to :obj:`True`).
 
     Attributes:
 
@@ -195,7 +195,7 @@ class RecipeCrash(StandardError):
 
        List of stack elements, with the most recent element (the one that
        caused the crash) at the end. Each stack element is a 
-       :class:`collections.namedtuple` with the following attributes:
+       :func:`collections.namedtuple` with the following attributes:
 
        .. attribute:: filename
  
@@ -301,28 +301,25 @@ class RecipeCrash(StandardError):
     def log(self, logger):
         '''Put the content of the crash into the log.
         '''
-        if self.elements:
-            log = logging.getLogger('%s.%s' 
-                                    % (logger.name, self.elements[-1].func))
-            log.error('Recipe crashed. Traceback (most recent call last):')
-            for e in self.elements:
-                log.error('  File "%s", %sin %s\n' % (
-                        e.filename, 
-                        'line %i, ' % e.line if e.line else '', 
-                        e.func))
-                if os.path.exists(e.filename) and e.line:
-                    log.error('    %s\n' 
-                              % file(e.filename).readlines()[e.line-1].strip())
-                if e.params:
-                    log.error('  Parameters:')
-                    for p, v in e.params.items():
-                        log.error('    %s = %s' % (p, v))
-                if e.localvars:
-                    log.error('  Local variables:')
-                    for p, v in e.localvars.items():
-                        log.error('    %s = %s' % (p, v))
-        else:
-            log = logging.getLogger(logger.name)
+        log = logging.getLogger('%s' % logger.name)
+        log.error('Recipe crashed. Traceback (most recent call last):')
+        for e in self.elements:
+            logc = logging.getLogger('%s.%s' % (logger.name, e.func))
+            logc.error('  File "%s", %sin %s\n' % (
+                    e.filename, 
+                    'line %i, ' % e.line if e.line else '', 
+                    e.func))
+            if os.path.exists(e.filename) and e.line:
+                logc.error('    %s\n' 
+                           % file(e.filename).readlines()[e.line-1].strip())
+            if e.params:
+                logc.error('  Parameters:')
+                for p, v in e.params.items():
+                    logc.error('    %s = %s' % (p, v))
+            if e.localvars:
+                logc.error('  Local variables:')
+                for p, v in e.localvars.items():
+                    logc.error('    %s = %s' % (p, v))
         log.error(RecipeCrash.signals.get(self.signal, 
                                           '%s: Unknown' % str(self.signal)))
 
